@@ -3,6 +3,7 @@ import jax.numpy as jnp
 import chex
 from typing import Tuple
 from ..strategy import Strategy
+from ..utils.eigen_decomp import full_eigen_decomp
 
 
 class CMA_ES(Strategy):
@@ -14,6 +15,7 @@ class CMA_ES(Strategy):
         assert 0 <= elite_ratio <= 1
         self.elite_ratio = elite_ratio
         self.elite_popsize = int(self.popsize * self.elite_ratio)
+        self.strategy_name = "CMA_ES"
 
     @property
     def params_strategy(self) -> chex.ArrayTree:
@@ -52,9 +54,7 @@ class CMA_ES(Strategy):
             1 / positive_sum * weights_prime,
             min_alpha / negative_sum * weights_prime,
         )
-        weights_truncated = jax.ops.index_update(
-            weights, jax.ops.index[self.elite_popsize :], 0
-        )
+        weights_truncated = weights.at[self.elite_popsize :].set(0)
 
         # lrate for cumulation of step-size control and rank-one update
         c_sigma = (mu_eff + 2) / (self.num_dims + mu_eff + 5)
@@ -77,14 +77,16 @@ class CMA_ES(Strategy):
             "mu_eff": mu_eff,
             "c_1": c_1,
             "c_mu": c_mu,
-            "c_m": 1,
+            "c_m": 1.0,
             "c_sigma": c_sigma,
             "d_sigma": d_sigma,
             "c_c": c_c,
             "chi_n": chi_n,
             "weights": weights,
-            "sigma_init": 1,
+            "sigma_init": 0.065,
             "weights_truncated": weights_truncated,
+            "init_min": 0.0,
+            "init_max": 0.0,
         }
         return params
 
@@ -114,7 +116,7 @@ class CMA_ES(Strategy):
         self, rng: chex.PRNGKey, state: chex.ArrayTree, params: chex.ArrayTree
     ) -> Tuple[chex.Array, chex.ArrayTree]:
         """`ask` for new parameter candidates to evaluate next."""
-        C, B, D = eigen_decomposition(state["C"], state["B"], state["D"])
+        C, B, D = full_eigen_decomp(state["C"], state["B"], state["D"])
         x = sample(
             rng,
             state["mean"],
@@ -160,7 +162,6 @@ class CMA_ES(Strategy):
         state["B"] = B
         state["D"] = D
         state["p_c"] = p_c
-        state["C"] = C
         state["sigma"] = sigma
         return state
 
@@ -188,7 +189,7 @@ def update_p_sigma(
     params: chex.ArrayTree,
 ) -> Tuple[chex.Array, chex.Array, chex.Array, None, None]:
     """Update evolution path for covariance matrix."""
-    C, B, D = eigen_decomposition(C, B, D)
+    C, B, D = full_eigen_decomp(C, B, D)
     C_2 = B.dot(jnp.diag(1 / D)).dot(B.T)  # C^(-1/2) = B D^(-1) B^T
     p_sigma_new = (1 - params["c_sigma"]) * p_sigma + jnp.sqrt(
         params["c_sigma"] * (2 - params["c_sigma"]) * params["mu_eff"]
@@ -278,72 +279,3 @@ def sample(
     y = jnp.swapaxes(y, 1, 0)
     x = mean + sigma * y  # ~ N(m, σ^2 C)
     return x
-
-
-def eigen_decomposition(
-    C: chex.Array, B: chex.Array, D: chex.Array
-) -> Tuple[chex.Array, chex.Array, chex.Array]:
-    """Perform eigendecomposition of covariance matrix."""
-    if B is not None and D is not None:
-        return C, B, D
-    C = (C + C.T) / 2
-    D2, B = jnp.linalg.eigh(C)
-    D = jnp.sqrt(jnp.where(D2 < 0, 1e-20, D2))
-    C = jnp.dot(jnp.dot(B, jnp.diag(D ** 2)), B.T)
-    return C, B, D
-
-
-# def check_initialization(params):
-#     """ Check lrates and other params of CMA-ES at initialization. """
-#     assert population_size > 0, "popsize must be non-zero positive value."
-#     assert n_dim > 1, "The dimension of mean must be larger than 1"
-#     assert sigma > 0, "sigma must be non-zero positive value"
-#     assert c_1 <= 1 - c_mu, "invalid lrate for the rank-one update"
-#     assert c_mu <= 1 - c_1, "invalid lrate for the rank-μ update"
-#     assert c_sigma < 1, "invalid lrate for cum. of step-size c."
-#     assert c_c <= 1, "invalid lrate for cum. of rank-one update"
-#     return
-#
-#
-# def check_termination(values, params, state):
-#     """ Check whether to terminate CMA-ES loop. """
-#     dC = jnp.diag(memory["C"])
-#     C, B, D = eigen_decomposition(memory["C"], memory["B"], memory["D"])
-#
-#     # Stop if generation fct values of recent generation is below thresh.
-#     if (memory["generation"] > params["min_generations"]
-#         and jnp.max(values) - jnp.min(values) < params["tol_fun"]):
-#         print("TERMINATE ----> Convergence/No progress in objective")
-#         return True
-#
-#     # Stop if std of normal distrib is smaller than tolx in all coordinates
-#     # and pc is smaller than tolx in all components.
-#     if jnp.all(memory["sigma"] * dC < params["tol_x"]) and np.all(
-#         memory["sigma"] * memory["p_c"] < params["tol_x"]):
-#         print("TERMINATE ----> Convergence/Search variance too small")
-#         return True
-#
-#     # Stop if detecting divergent behavior.
-#     if memory["sigma"] * jnp.max(D) > params["tol_x_up"]:
-#         print("TERMINATE ----> Stepsize sigma exploded")
-#         return True
-#
-#     # No effect coordinates: stop if adding 0.2-standard deviations
-#     # in any single coordinate does not change m.
-#     if jnp.any(memory["mean"] == memory["mean"] + (0.2 * memory["sigma"] * jnp.sqrt(dC))):
-#         print("TERMINATE ----> No effect when adding std to mean")
-#         return True
-#
-#     # No effect axis: stop if adding 0.1-standard deviation vector in
-#     # any principal axis direction of C does not change m.
-#     if jnp.all(memory["mean"] == memory["mean"] + (0.1 * memory["sigma"]
-#                                 * D[0] * B[:, 0])):
-#         print("TERMINATE ----> No effect when adding std to mean")
-#         return True
-#
-#     # Stop if the condition number of the covariance matrix exceeds 1e14.
-#     condition_cov = jnp.max(D) / jnp.min(D)
-#     if condition_cov > params["tol_condition_C"]:
-#         print("TERMINATE ----> C condition number exploded")
-#         return True
-#     return False
